class Danila_vagon_text_recognize_params:
    def __init__(self, vagon_text_recognize_version, vagon_text_recognize_size_h, vagon_text_recognize_size_w):
        self.vagon_text_recognize_version = vagon_text_recognize_version
        self.vagon_text_recognize_size_h = vagon_text_recognize_size_h
        self.vagon_text_recognize_size_w = vagon_text_recognize_size_w


import abc

class Prod_coefficients:
    def __init__(self, number_coefficients, prod_coefficients, year_coefficients):
        self.number_coefficients = number_coefficients
        self.prod_coefficients = prod_coefficients
        self.year_coefficients = year_coefficients

class Text_coefficients:
    def __init__(self, length, height, width):
        self.length = length
        self.height = height
        self.width = width

class Danila_balka_text_recognize_model(abc.ABC):
    @abc.abstractmethod
    def get_model(self, yolo_path): pass

    @abc.abstractmethod
    def get_prod_coefficients(self, balka_prod): pass

    @abc.abstractmethod
    def get_model_altai(self, yolo_path): pass

    @abc.abstractmethod
    def get_altai_prod_coefficients(self): pass



class Danila_rama_text_recognize_prod_redone:

    def __init__(self, danila_rama_text_recognize):
        self.danila_rama_text_recognize = danila_rama_text_recognize

    def rama_text_recognize(self, rama_prod, img_cut, image_text_areas):
        return self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, image_text_areas)

    def get_number_length(self, rama_prod):
        return self.danila_rama_text_recognize.get_number_length(rama_prod)



from data.neuro.Detail_classify_class import Detail_classify_class
from data.neuro.local_models import *
from data.neuro.models import DETAIL_CLASSIFY_MODEL_ADDRESS, DETAIL_CLASSIFY_MODEL_ADDRESS_2
from data.result.Text_cut_recognize_result import Text_cut_recognize_result
from data.result.Text_recognize_result import Text_recognize_result


class Danila_detail_CNN():
    def __init__(self, local, CNN_model_version):
        if local:
            if CNN_model_version == 1:
                detail_classify_model_path = LOCAL_DETAIL_CLASSIFY_MODEL_ADDRESS
            else:
                detail_classify_model_path = LOCAL_DETAIL_CLASSIFY_MODEL_ADDRESS_2
            print('reading and loading - DETAIL_CLASSIFY_MODEL')
            self.detail_classify_model = Detail_classify_class(local, detail_classify_model_path)
        else:
            if CNN_model_version == 1:
                detail_classify_model_path = DETAIL_CLASSIFY_MODEL_ADDRESS
            else:
                detail_classify_model_path = DETAIL_CLASSIFY_MODEL_ADDRESS_2
            print('reading and loading - DETAIL_CLASSIFY_MODEL')
            self.detail_classify_model = Detail_classify_class(local, detail_classify_model_path)


    def detail_classify(self, img):
        class_detail_conf = self.detail_classify_model.classify(img)
        detail = Text_recognize_result(
            Text_cut_recognize_result(class_detail_conf.class_detail.name, class_detail_conf.conf))
        return detail



from data.neuro.Rama_prod_classify_class import Rama_prod_classify_class
from data.neuro.Rama_prod_classify_dop_detect import Rama_prod_classify_dop_detect
from data.neuro.local_models import *
from data.neuro.models import RAMA_PROD_CLASSIFY_MODEL_ADDRESS, RAMA_PROD_CLASSIFY_MODEL_ADDRESS_2, \
    RAMA_PROD_CLASSIFY_MODEL_ADDRESS_3


class Danila_rama_classify_prod_dop_detect:
    def __init__(self, local, yolov5_dir, rama_classify_model):
        yolo_path = yolov5_dir
        if local:
            if rama_classify_model == 3:
                rama_prod_classify_model_path = LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_3
                self.size = 480
                self.dop_size = 256
            elif rama_classify_model == 4:
                rama_prod_classify_model_path = LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_4
                self.size = 608
                self.dop_size = 224
            elif rama_classify_model == 5:
                rama_prod_classify_model_path = LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_5
                self.size = 608
                self.dop_size = 224
        else:
            if rama_classify_model == 5:
                rama_prod_classify_model_path = LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_5
                self.size = 608
        print('reading and loading - RAMA_PROD_CLASSIFY_MODEL')
        self.rama_prod_classify_model = Rama_prod_classify_dop_detect(local, rama_prod_classify_model_path, yolo_path, LOCAL_RAMA_DETECT_MODEL_ADDRESS)

    def rama_classify(self, img):
        rama_prod_conf = self.rama_prod_classify_model.classify(img, self.size, self.dop_size)
        return rama_prod_conf


from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize_model import *
from data.neuro.balka_text_recognize_altai_yolo import Balka_Text_Recognize_altai_yolo
from data.neuro.balka_text_recognize_yolo import Balka_Text_Recognize_yolo
from data.neuro.local_models import *
from data.result.balka_prod import Balka_Prod


class Danila_balka_text_recognize_model_6(Danila_balka_text_recognize_model):
    def get_model(self, yolo_path):
        return Balka_Text_Recognize_yolo(model_path= LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_4, yolo_path=yolo_path)

    def get_model_altai(self, yolo_path):
        return Balka_Text_Recognize_altai_yolo(model_path=LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_4,
                                               yolo_path=yolo_path,
                                               prod_coefficients=self.get_altai_prod_coefficients())
    def get_model_cut_first_last(self, yolo_path):
        return Balka_Text_Recognize_yolo(model_path=LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_4,
                                         yolo_path=yolo_path, cut_first_last=True)

    def get_prod_coefficients(self, balka_prod):

        if balka_prod == Balka_Prod.begickaya:
            return Prod_coefficients(
                number_coefficients=Text_coefficients(
                    length=5, height=160, width=160
                ),
                prod_coefficients=Text_coefficients(
                    length=2, height=96, width=96
                ),
                year_coefficients=Text_coefficients(
                    length=2, height=64, width=64
                ),
            )
        elif balka_prod == Balka_Prod.promlit:
            return Prod_coefficients(
                number_coefficients=Text_coefficients(
                    length=5, height=160, width=160
                ),
                prod_coefficients=Text_coefficients(
                    length=2, height=128, width=128
                ),
                year_coefficients=Text_coefficients(
                    length=2, height=64, width=64
                ),
            )
        elif balka_prod == Balka_Prod.ruzhimmash:
            return Prod_coefficients(
                number_coefficients=Text_coefficients(
                    length=5, height=128, width=128
                ),
                prod_coefficients=Text_coefficients(
                    length=4, height=128, width=128
                ),
                year_coefficients=Text_coefficients(
                    length=2, height=64, width=64
                ),
            )
        elif balka_prod == Balka_Prod.tihvin:
            return Prod_coefficients(
                number_coefficients=Text_coefficients(
                    length=5, height=128, width=128
                ),
                prod_coefficients=Text_coefficients(
                    length=4, height=128, width=128
                ),
                year_coefficients=Text_coefficients(
                    length=2, height=64, width=64
                ),
            )

    def get_altai_prod_coefficients(self):

        return [
                Prod_coefficients(
                    number_coefficients=Text_coefficients(
                        length=5, height=96, width=96
                    ),
                    prod_coefficients=Text_coefficients(
                        length=4, height=160, width=160
                    ),
                    year_coefficients=Text_coefficients(
                        length=2, height=96, width=96
                    )
                ),
                Prod_coefficients(
                    number_coefficients=Text_coefficients(
                        length=6, height=160, width=160
                    ),
                    prod_coefficients=Text_coefficients(
                        length=4, height=160, width=160
                    ),
                    year_coefficients=Text_coefficients(
                        length=2, height=96, width=96
                    )
                )
            ]


from enum import Enum


class Balka_Prod(Enum):
    altai = 0
    begickaya = 1
    promlit = 2
    ruzhimmash = 3
    tihvin = 4
    no_balka = 5

class Balka_Prod_Conf:

    def __init__(self, balka_prod, conf):
        self.balka_prod = balka_prod
        self.conf = conf


class Danila_balka_text_recognize_prod_redone:

    def __init__(self, danila_balka_text_recognize):
        self.danila_balka_text_recognize = danila_balka_text_recognize

    def balka_text_recognize(self, balka_prod, img_cut, image_text_areas):
        return self.danila_balka_text_recognize.balka_text_recognize(balka_prod, img_cut, image_text_areas)




from danila.danila_rama.danila_rama_base import Danila_rama_base
from danila.danila_rama.danila_rama_redone import Danila_rama_redone


class Danila_rama:
    def __init__(self, local, yolov5_dir, danila_rama_params):
        if danila_rama_params.danila_rama_text_recognize_params.prod_redone:
            self.danila_rama = Danila_rama_redone(local, yolov5_dir=yolov5_dir, danila_rama_params=danila_rama_params)
        else:
            self.danila_rama = Danila_rama_base(local, yolov5_dir, danila_rama_params)

    def rama_classify(self,img, detail):
        return self.danila_rama.rama_classify(img, detail)

    def rama_text_detect(self, img):
        return self.danila_rama.rama_text_detect(img)

    def rama_text_recognize(self, img, detail):
        return self.danila_rama.rama_text_recognize(img, detail)


from data.result.Class_text import Class_text




class Danila_balka_text_recognize_work:
    def __init__(self, text_recognize_model, prod_coefficients):
        self.text_recognize_model = text_recognize_model
        self.prod_coefficients = prod_coefficients

    def balka_text_recognize(self, _balka_prod, img_cut, image_text_areas):
        label_area = self.text_recognize_model.work_image_cut(
            _balka_prod, image_text_areas, img_cut,
            self.prod_coefficients.number_coefficients.length,
            self.prod_coefficients.number_coefficients.height,
            self.prod_coefficients.number_coefficients.width,
            self.prod_coefficients.prod_coefficients.length,
            self.prod_coefficients.prod_coefficients.height,
            self.prod_coefficients.prod_coefficients.width,
            self.prod_coefficients.year_coefficients.length,
            self.prod_coefficients.year_coefficients.height,
            self.prod_coefficients.year_coefficients.width
        )
        res_labels = {}
        (number_text, number_conf) = label_area.labels[Class_text.number]
        res_labels['number'] = (number_text, number_conf)
        (year_text, year_conf) = label_area.labels[Class_text.year]
        res_labels['year'] = (year_text, year_conf)
        (prod_text, prod_conf) = label_area.labels[Class_text.prod]
        res_labels['prod'] = (prod_text, prod_conf)
        return res_labels



import hashlib
import json
import os
from datetime import datetime

import cv2

import torch

from data.neuro.letters_in_image import Letters_In_Image
from data.result.Class_text import Class_text
from data.result.Class_text_altai import Class_text_altai
from data.result.Label_area import Label_area


class Balka_Text_Recognize_altai_yolo:

    def __init__(self, model_path, yolo_path, prod_coefficients):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        self.model = torch.hub.load(yolo_path, 'custom', model_path, source='local')
        self.prod_coefficients = prod_coefficients
        # for every text_class try to recognize text from all areas of text_class, number_params is depends on class and prod, returns string

    def work_image_cuts(self, number_image_cuts, l, h, w):
        """for every text_class try to recognize text from all areas of text_class, number_params is depends on class and prod, returns string """
        (str,conf) = ('', 0.0)
        for number_image_cut in number_image_cuts:
            # cv2.imwrite('data/neuro_classes/cut.jpg', number_image_cut)
            (cur_str, cur_conf) = self.work_img_word(number_image_cut, l, h, w)
            if len(cur_str) == l:
                if cur_conf > conf:
                    (str, conf) = (cur_str, cur_conf)
            elif len(cur_str) > len(str):
                (str, conf) = (cur_str, cur_conf)
            elif (len(cur_str) == len(str)) & (cur_conf > conf):
                (str, conf) = (cur_str, cur_conf)
        return (str, round(conf,2))

        # main_method takes all image_text_areas from image_rama_cut and recognize text

    def prepare_cuts(self, image_cuts_2_half_balkas, class_text, has_prod = True):
        result_img_cuts = []
        if class_text == Class_text.number:
            if len(image_cuts_2_half_balkas) == 2:
                for (number_img_cut) in image_cuts_2_half_balkas[0]:
                    result_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_COUNTERCLOCKWISE))
                for (number_img_cut) in image_cuts_2_half_balkas[1]:
                    result_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                return result_img_cuts
            elif len(image_cuts_2_half_balkas) == 1:
                if has_prod:
                    for (number_img_cut) in image_cuts_2_half_balkas[0]:
                        result_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_COUNTERCLOCKWISE))
                else:
                    for (number_img_cut) in image_cuts_2_half_balkas[0]:
                        result_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                return result_img_cuts
            else:
                return result_img_cuts
        elif class_text == Class_text.prod:
            for image_cuts in image_cuts_2_half_balkas:
                for image_cut in image_cuts:
                    result_img_cuts.append(image_cut)
            return result_img_cuts
        elif class_text == Class_text.year:
            for image_cuts in image_cuts_2_half_balkas:
                for image_cut in image_cuts:
                    result_img_cuts.append(image_cut)
            return result_img_cuts


    def work_image_cut(self, image_text_areas_2_half_balkas, image_balka_cuts):
        """main_method takes all image_text_areas from image_rama_cut and recognize text"""
        # number
        result_number_img_cuts = []
        result_prod_image_cuts = []
        result_year_image_cuts = []
        if len(image_text_areas_2_half_balkas) == 2:
            seria_left = (len(image_text_areas_2_half_balkas[0].areas[Class_text_altai.seria]) > 0)
            seria_right = (len(image_text_areas_2_half_balkas[1].areas[Class_text_altai.seria]) > 0)
            if (not seria_left) and seria_right:
                number_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.number]
                image_rama_cut = image_balka_cuts[0]
                number_image_cuts = self.make_cuts(image_rama_cut, number_image_rects)
                for (number_img_cut) in number_image_cuts:
                    result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                pattern_index = 1
            elif seria_left and (not(seria_right)):
                i = 0
                number_image_cuts_2_half_balkas = []
                for image_text_areas in image_text_areas_2_half_balkas:
                    number_image_rects = image_text_areas.areas[Class_text_altai.number]
                    image_rama_cut = image_balka_cuts[i]
                    number_image_cuts_2_half_balkas.append(self.make_cuts(image_rama_cut, number_image_rects))
                    i += 1
                for (number_img_cut) in number_image_cuts_2_half_balkas[0]:
                    result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_COUNTERCLOCKWISE))
                for (number_img_cut) in number_image_cuts_2_half_balkas[1]:
                    result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                pattern_index = 0
            else:
                if len(image_text_areas_2_half_balkas[1].areas[Class_text_altai.sran]) > 0:
                    number_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.number]
                    image_rama_cut = image_balka_cuts[0]
                    number_image_cuts = self.make_cuts(image_rama_cut, number_image_rects)
                    for (number_img_cut) in number_image_cuts:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                    pattern_index = 1
                else:
                    i = 0
                    number_image_cuts_2_half_balkas = []
                    for image_text_areas in image_text_areas_2_half_balkas:
                        number_image_rects = image_text_areas.areas[Class_text_altai.number]
                        image_rama_cut = image_balka_cuts[i]
                        number_image_cuts_2_half_balkas.append(self.make_cuts(image_rama_cut, number_image_rects))
                        i += 1
                    for (number_img_cut) in number_image_cuts_2_half_balkas[0]:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_COUNTERCLOCKWISE))
                    for (number_img_cut) in number_image_cuts_2_half_balkas[1]:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                    pattern_index = 0
            prod_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.prod]
            image_rama_cut = image_balka_cuts[0]
            result_prod_image_cuts = self.make_cuts(image_rama_cut, prod_image_rects)
            year_image_rects = image_text_areas_2_half_balkas[1].areas[Class_text_altai.year]
            image_rama_cut = image_balka_cuts[1]
            result_year_image_cuts = self.make_cuts(image_rama_cut, year_image_rects)
        else:
            has_prod = (len(image_text_areas_2_half_balkas[0].areas[Class_text_altai.prod]) > 0)
            has_year = (len(image_text_areas_2_half_balkas[0].areas[Class_text_altai.year]) > 0)
            has_seria = (len(image_text_areas_2_half_balkas[0].areas[Class_text_altai.seria]) > 0)
            has_sran = (len(image_text_areas_2_half_balkas[0].areas[Class_text_altai.sran]) > 0)
            if has_prod:
                if has_seria:
                    number_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.number]
                    image_rama_cut = image_balka_cuts[0]
                    number_image_cuts = self.make_cuts(image_rama_cut, number_image_rects)
                    for (number_img_cut) in number_image_cuts:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_COUNTERCLOCKWISE))
                    pattern_index = 0
                    prod_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.prod]
                    image_rama_cut = image_balka_cuts[0]
                    result_prod_image_cuts = self.make_cuts(image_rama_cut, prod_image_rects)
                else:
                    number_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.number]
                    image_rama_cut = image_balka_cuts[0]
                    number_image_cuts = self.make_cuts(image_rama_cut, number_image_rects)
                    for (number_img_cut) in number_image_cuts:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                    pattern_index = 1
                    prod_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.prod]
                    image_rama_cut = image_balka_cuts[0]
                    result_prod_image_cuts = self.make_cuts(image_rama_cut, prod_image_rects)
            else:
                if has_seria:
                    year_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.year]
                    image_rama_cut = image_balka_cuts[0]
                    result_year_image_cuts = self.make_cuts(image_rama_cut, year_image_rects)
                    pattern_index = 1
                else:
                    number_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.number]
                    image_rama_cut = image_balka_cuts[0]
                    number_image_cuts = self.make_cuts(image_rama_cut, number_image_rects)
                    for (number_img_cut) in number_image_cuts:
                        result_number_img_cuts.append(cv2.rotate(number_img_cut, cv2.ROTATE_90_CLOCKWISE))
                    pattern_index = 0
                    year_image_rects = image_text_areas_2_half_balkas[0].areas[Class_text_altai.year]
                    image_rama_cut = image_balka_cuts[0]
                    result_year_image_cuts = self.make_cuts(image_rama_cut, year_image_rects)
        p_c = self.prod_coefficients[pattern_index]
        #  for debug - comment in production
        # i = 0
        # for result_number_img_cut in result_number_img_cuts:
        #     cv2.namedWindow('number'+str(i), cv2.WINDOW_AUTOSIZE)
        #     cv2.imshow('number'+str(i), result_number_img_cut)
        #     i+=1
        # i = 0
        # for result_prod_image_cut in result_prod_image_cuts:
        #     cv2.namedWindow('prod' + str(i), cv2.WINDOW_AUTOSIZE)
        #     cv2.imshow('prod' + str(i), result_prod_image_cut)
        #     i+=1
        # i = 0
        # for result_year_image_cut in result_year_image_cuts:
        #     cv2.namedWindow('year' + str(i), cv2.WINDOW_AUTOSIZE)
        #     cv2.imshow('year' + str(i), result_year_image_cut)
        #     i += 1
        # cv2.waitKey(0)
        # cv2.destroyAllWindows()
        (number, number_conf) = self.work_image_cuts(result_number_img_cuts,
                                                     p_c.number_coefficients.length,
                                                     p_c.number_coefficients.height,
                                                     p_c.number_coefficients.width
                                                     )
        (prod, prod_conf) = self.work_image_cuts(result_prod_image_cuts,
                                                     p_c.prod_coefficients.length,
                                                     p_c.prod_coefficients.height,
                                                     p_c.prod_coefficients.width
                                                     )
        (year, year_conf) = self.work_image_cuts(result_year_image_cuts,
                                                     p_c.year_coefficients.length,
                                                     p_c.year_coefficients.height,
                                                     p_c.year_coefficients.width
                                                     )
        label_area = Label_area()
        label_area.labels[Class_text.number] = (number, number_conf)
        label_area.labels[Class_text.prod] = (prod, prod_conf)
        label_area.labels[Class_text.year] = (year, year_conf)
        return label_area

    def work_img_word(self, number_image_cut, l, number_h, number_w):
        h, w = number_image_cut.shape[:2]
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        img_cut_path = 'cut_text_img' + hash_str + '.jpg'
        cv2.imwrite(img_cut_path, number_image_cut)
        self.model.max_det = l
        results = self.model(img_cut_path, size=(number_h, number_w))
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        img_letters = Letters_In_Image.get_letters_in_image_from_yolo_json(res2)
        img_letters.sort_letters()
        img_letters.delete_intersections()
        os.remove(img_cut_path)
        return img_letters.make_word(), img_letters.get_avg_conf()

    # cut text_areas imgs for each Rect from rect_array returns openCv imgs list
    def make_cuts(self, img_balka_cut, rect_array):
        """cut text_areas imgs for each Rect from rect_array returns openCv imgs list"""
        number_image_cuts = []
        for rect in rect_array:
            number_image_cuts.append(img_balka_cut[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        return number_image_cuts
        



from danila.danila_balka.danila_balka_base import Danila_balka_base
from danila.danila_balka.danila_balka_redone import Danila_balka_redone


class Danila_balka:
    def __init__(self, yolov5_dir, danila_balka_params):
        if danila_balka_params.danila_balka_text_recognize_params.prod_redone:
            self.danila_balka = Danila_balka_redone(yolov5_dir, danila_balka_params)
        else:
            self.danila_balka = Danila_balka_base(yolov5_dir, danila_balka_params)

    def balka_classify(self,img, detail):
        return self.danila_balka.balka_classify(img, detail)

    def balka_text_detect(self, img):
        return self.danila_balka.balka_text_detect(img)

    def balka_text_recognize(self, img, detail):
        return self.danila_balka.balka_text_recognize(img, detail)


from danila.danila_rama.danila_rama_classify.danila_rama_classify_prod import Danila_rama_classify_prod
from danila.danila_rama.danila_rama_classify.danila_rama_classify_prod_dop_detect import \
    Danila_rama_classify_prod_dop_detect


class Danila_rama_classify:
    def __init__(self, local, yolov5_dir, danila_rama_classify_params):
        self.danila_rama_classify_params = danila_rama_classify_params
        if self.danila_rama_classify_params.rama_classify_version == 1:
            self.danila_rama_classify = Danila_rama_classify_prod(local, yolov5_dir, self.danila_rama_classify_params.rama_classify_model)
        elif self.danila_rama_classify_params.rama_classify_version == 2:
            self.danila_rama_classify = Danila_rama_classify_prod_dop_detect(local, yolov5_dir, self.danila_rama_classify_params.rama_classify_model - 5)

    def rama_classify(self, img):
        return self.danila_rama_classify.rama_classify(img)




from data.result.Class_text import Class_text
from data.result.Rect import Rect


class Danila_rama_text_recognize_number_shift:
    def __init__(self, danila_rama_text_recognize):
        self.danila_rama_text_recognize = danila_rama_text_recognize

    def get_number_length(self, rama_prod):
        return self.danila_rama_text_recognize.get_number_length(rama_prod)

    def rama_text_recognize(self, rama_prod, img_cut, image_text_areas):
        res_labels = self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, image_text_areas)
        (number_text, _) = res_labels['number']
        if len(number_text) == self.danila_rama_text_recognize.get_number_length(rama_prod):
            return res_labels
        else:
            number_image_text_areas = image_text_areas.areas[Class_text.number]
            image_text_areas_min = []
            image_text_areas_max = []
            for number_image_text_area in number_image_text_areas:
                w = number_image_text_area.xmax - number_image_text_area.xmin
                if (number_image_text_area.xmin - (w // 2)) < 0:
                    new_xmin = 0
                else:
                    new_xmin = number_image_text_area.xmin - (w // 2)
                rect_min = Rect(xmin=new_xmin, xmax=number_image_text_area.xmax, ymin=number_image_text_area.ymin,
                                ymax=number_image_text_area.ymax)
                image_text_areas_min.append(rect_min)
                new_xmax = number_image_text_area.xmax + w // 2
                rect_max = Rect(xmin=number_image_text_area.xmin, xmax=new_xmax, ymin=number_image_text_area.ymin,
                                ymax=number_image_text_area.ymax)
                image_text_areas_max.append(rect_max)
            image_text_areas.areas[Class_text.number] = image_text_areas_min
            res_labels_min = self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, image_text_areas)
            (number_text_min, _) = res_labels_min['number']
            if len(number_text_min) == self.danila_rama_text_recognize.get_number_length(rama_prod):
                return res_labels_min
            else:
                image_text_areas.areas[Class_text.number] = image_text_areas_max
                res_labels_max = self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, image_text_areas)
                (number_text_max, _) = res_labels_max['number']
                if len(number_text_max) == self.danila_rama_text_recognize.get_number_length(rama_prod):
                    return res_labels_max
                else:
                    return res_labels


class Danila_balka_params:
    def __init__(self, danila_balka_classify_params, danila_balka_detect_params, danila_balka_text_recognize_params):
        self.danila_balka_classify_params = danila_balka_classify_params
        self.danila_balka_text_detect_params = danila_balka_detect_params
        self.danila_balka_text_recognize_params = danila_balka_text_recognize_params


from danila.danila_detail.Danila_detail_CNN import Danila_detail_CNN
from danila.danila_detail.Danila_detail_yolo_dop_yolo_dop_CNN import Danila_detail_yolo_dop_yolo_dop_CNN


class Danila_detail():
    def __init__(self, local, danila_detail_params, yolo_dir = ''):
        self.danila_detail_params = danila_detail_params
        if (danila_detail_params.model_version == 1):
            self.danila_detail = Danila_detail_CNN(local, danila_detail_params.CNN_model_version)
        elif (danila_detail_params.model_version == 2):
            self.danila_detail = Danila_detail_yolo_dop_yolo_dop_CNN(local, yolo_dir, danila_detail_params.CNN_model_version)



    def detail_classify(self, img):
        return self.danila_detail.detail_classify(img)




LOCAL_RAMA_BEGICKAYA_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_begickaya_detect.pt'
LOCAL_RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_ruzhimmash_detect_weights.pt'
LOCAL_RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS_2 = 'models/rama_detect/rama_ruzhimmash_detect_2.pt'
LOCAL_RAMA_ALTAI_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_altai_detect_300.pt'
LOCAL_RAMA_ALTAI_DETECT_MODEL_ADDRESS_2 = 'models/rama_detect/rama_altai_detect_400.pt'
LOCAL_RAMA_BALAKOVO_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_balakovo_detect.pt'
LOCAL_RAMA_PROMLIT_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_promlit_detect_300.pt'
LOCAL_RAMA_PROMLIT_DETECT_MODEL_ADDRESS_2 = 'models/rama_detect/rama_promlit_detect_600.pt'
LOCAL_RAMA_TIHVIN_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_tihvin_detect.pt'
LOCAL_RAMA_URALVAGON_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_uralvagon_detect.pt'
LOCAL_RAMA_DETECT_MODEL_ADDRESS = 'models/rama_detect/rama_detect.pt'

LOCAL_RAMA_ALTAI_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_altai_text_detect.pt'
LOCAL_RAMA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/rama_text_detect/rama_altai_text_detect_400.pt'
LOCAL_RAMA_BALAKOVO_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_balakovo_text_detect.pt'
LOCAL_RAMA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_begickaya_text_detect.pt'
LOCAL_RAMA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_promlit_text_detect.pt'
LOCAL_RAMA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/rama_text_detect/rama_promlit_text_detect_600.pt'
LOCAL_RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_ruzhimmash_text_detect.pt'
LOCAL_RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/rama_text_detect/rama_ruzhimmash_text_detect_2.pt'
LOCAL_RAMA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_tihvin_text_detect.pt'
LOCAL_RAMA_URALVAGON_TEXT_DETECT_MODEL_ADDRESS = 'models/rama_text_detect/rama_uralvagon_text_detect.pt'

LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS = 'models/rama_text_recognize/rama_b_r_text_recognize.pt' # 11000_b_11000_r
LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_2 = 'models/rama_text_recognize/rama_a_b_r_text_recognize.pt' # 5000_a_11000_b_11000_r
LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_3 = 'models/rama_text_recognize/rama_a_b_p_r_t_text_recognize.pt' # 5000_a_11000_b_2000_p_11000_r_5000_t
LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_4 = 'models/rama_text_recognize/rama_balka_all_text_recognize.pt'
LOCAL_RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_5 = 'models/rama_text_recognize/rama_balka_all_2_text_recognize.pt'

LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/8FI8TaTwaCfbSA'
LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_2 = 'models/rama_classify/model_3500.pt'
LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_3 = 'models/rama_classify/model_4000.pt'
LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_4 = 'models/rama_classify/model_4200.pt'
LOCAL_RAMA_PROD_CLASSIFY_MODEL_ADDRESS_5 = 'models/rama_classify/model_4200_n.pt'
LOCAL_VAGON_NUMBER_DETECT_MODEL_ADDRESS = 'models/vagon_number_detect/vagon_number_detect.pt'
LOCAL_VAGON_NUMBER_DETECT_MODEL_ADDRESS_2 = 'models/vagon_number_detect/vagon_number_detect_2.pt'

LOCAL_VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS = 'models/vagon_number_recognize/06_10_6000_im/exp39/weights/text_recognize_yolo.pt'
LOCAL_VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_2 = 'models/vagon_number_recognize/06_16_9000_im/exp41/weights/text_recognize_yolo.pt'
LOCAL_VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_3 = 'models/vagon_number_recognize/06_16_22000_im/exp40/weights/text_recognize_yolo.pt'
LOCAL_VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_4 = 'models/vagon_number_recognize/06_20_9500_im/exp43/weights/text_recognize_yolo.pt'

LOCAL_BALKA_CLASSIFY_MODEL_ADDRESS = 'models/balka_classify/balka_classify_1500.pt'
LOCAL_BALKA_CLASSIFY_MODEL_ADDRESS_2 = 'models/balka_classify/balka_classify_10000.pt'

LOCAL_BALKA_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_detect/balka_detect.pt'
LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_altai_detect/balka_altai_detect.pt'
LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS_2 = 'models/balka_detect/balka_altai_detect/balka_altai_detect_1000.pt'
LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS_3 = 'models/balka_detect/balka_altai_detect/balka_altai_detect_2000.pt'
LOCAL_BALKA_BEGICKAYA_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_begickaya_detect/balka_begickaya_detect.pt'
LOCAL_BALKA_PROMLIT_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_promlit_detect/balka_detect.pt'
LOCAL_BALKA_RUZHIMMASH_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_ruzhimmash_detect/balka_detect.pt'
LOCAL_BALKA_TIHVIN_DETECT_MODEL_ADDRESS = 'models/balka_detect/balka_tihvin_detect/balka_detect.pt'

LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS = 'models/balka_text_detect/balka_altai_text_detect/balka_altai_text_detect.pt'
LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/balka_text_detect/balka_altai_text_detect/balka_altai_text_detect_2000.pt'
LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_3 = 'models/balka_text_detect/balka_altai_text_detect/balka_altai_5_text_detect.pt'
LOCAL_BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS = 'models/balka_text_detect/balka_begickaya_text_detect/balka_begickaya_text_detect.pt'
LOCAL_BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/balka_text_detect/balka_begickaya_text_detect/balka_begickaya_text_detect_2.pt'
LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS = 'models/balka_text_detect/balka_promlit_text_detect/balka_promlit_text_detect.pt'
LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/balka_text_detect/balka_promlit_text_detect/balka_promlit_text_detect_2.pt'
LOCAL_BALKA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS = 'models/balka_text_detect/balka_ruzhimmash_text_detect/balka_ruzhimmash_text_detect.pt'
LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS = 'models/balka_text_detect/balka_tihvin_text_detect/balka_tihvin_text_detect.pt'
LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS_2 = 'models/balka_text_detect/balka_tihvin_text_detect/balka_tihvin_4_text_detect.pt'

LOCAL_DETAIL_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/Xamk_OYzYAbGhg'
LOCAL_DETAIL_CLASSIFY_MODEL_ADDRESS_2 = 'models/detail_classify/detail_classify_balka/detail_classify_model.h5'
LOCAL_DETAIL_CLASSIFY_YOLO_MODEL_ADDRESS = 'models/detail_classify/detail_classify_4500/detail_classify.pt'
LOCAL_DETAIL_CLASSIFY_BALKA_MODEL_ADDRESS = LOCAL_BALKA_PROMLIT_DETECT_MODEL_ADDRESS
LOCAL_DETAIL_CLASSIFY_RAMA_MODEL_ADDRESS = LOCAL_RAMA_BALAKOVO_DETECT_MODEL_ADDRESS
LOCAL_DETAIL_CLASSIFY_VAGON_MODEL_ADDRESS = LOCAL_VAGON_NUMBER_DETECT_MODEL_ADDRESS




import json
import os
import zipfile

import cv2
import requests
from urllib.parse import urlencode
import torch
import hashlib
from datetime import datetime
from data.neuro.letters_in_image import Letters_In_Image
from data.neuro.objs_in_image import Objs_In_Image
from data.result.Class_text import Class_text
from data.result.Label_area import Label_area
from data.result.Rama_prod import Rama_Prod, Rama_Prod_Conf


class Rama_prod_classify_dop_detect:

    def __init__(self, local, model_path, yolo_path, rama_detect_model):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        if local:
            self.model = torch.hub.load(yolo_path, 'custom', model_path, source='local')
            self.rama_detect_model = torch.hub.load(yolo_path, 'custom', rama_detect_model, source='local')
        else:
            self.model = torch.hub.load(yolo_path, 'custom', model_path, source='local')
            self.rama_detect_model = torch.hub.load(yolo_path, 'custom', rama_detect_model, source='local')

        # for every text_class try to recognize text from all areas of text_class, number_params is depends on class and prod, returns string


    def work_image(self, img, size, dop_size):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        img_name = 'img' + hash_str + '.jpg'
        cv2.imwrite(img_name, img)
        results = self.model([img_name], size = size)
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        if len(res2) == 0:
            dop_rama_results = self.rama_detect_model([img_name], size=512)
            dop_rama_json_res = dop_rama_results.pandas().xyxy[0].to_json(orient="records")
            dop_rama_res2 = json.loads(dop_rama_json_res)
            if len(dop_rama_res2) == 0:
                result = Rama_Prod_Conf(Rama_Prod(7), 0.0)
            else:
                rama_img = img[int(dop_rama_res2[0]['ymin']):int(dop_rama_res2[0]['ymax']),
                           int(dop_rama_res2[0]['xmin']):int(dop_rama_res2[0]['xmax'])]
                cv2.imwrite('rama_' + img_name, rama_img)
                prod_results = self.model(['rama_' + img_name], size=dop_size)
                prod_json_res = prod_results.pandas().xyxy[0].to_json(orient="records")
                prod_res2 = json.loads(prod_json_res)
                os.remove('rama_' + img_name)
                if len(prod_res2) == 0:
                    result = Rama_Prod_Conf(Rama_Prod(7), 0.0)
                else:
                    res_index = prod_res2[0]['class']
                    res_conf = prod_res2[0]['confidence']
                    result = Rama_Prod_Conf(Rama_Prod(res_index), res_conf)
        else:
            res_index = res2[0]['class']
            res_conf = res2[0]['confidence']
            result = Rama_Prod_Conf(Rama_Prod(res_index), res_conf)
        os.remove(img_name)
        return result

    def classify(self, img, size_p, dop_size):
        self.model.max_det = 1
        rama_prod_conf = self.work_image(img, size_p, dop_size)
        return rama_prod_conf


import hashlib
import os
from datetime import datetime

import cv2

from data.neuro.Balka_detect_class import Balka_detect_class
from data.neuro.balka_text_detect_altai_class import Balka_text_detect_altai_class
from data.neuro.balka_text_detect_class import Balka_text_detect_class
from data.neuro.local_models import *
from data.neuro.models import *
from data.result import Rama_prod, balka_prod


class Danila_balka_text_detect_base:
    def __init__(self, yolov5_dir, balka_detect_version, balka_text_detect_version):
        self.balka_detect_objects = {
            balka_prod.Balka_Prod.altai: None,
            balka_prod.Balka_Prod.begickaya: None,
            balka_prod.Balka_Prod.promlit: None,
            balka_prod.Balka_Prod.ruzhimmash: None,
            balka_prod.Balka_Prod.tihvin: None
        }
        print('reading and loading - BALKA_BEGICKAYA_DETECT_MODEL')
        self.balka_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_detect_class(
            LOCAL_BALKA_BEGICKAYA_DETECT_MODEL_ADDRESS, yolov5_dir)

        print('reading and loading - BALKA_PROMLIT_DETECT_MODEL')
        self.balka_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_detect_class(
            LOCAL_BALKA_PROMLIT_DETECT_MODEL_ADDRESS, yolov5_dir)

        print('reading and loading - BALKA_RUZHIMMASH_DETECT_MODEL')
        self.balka_detect_objects[balka_prod.Balka_Prod.ruzhimmash] = Balka_detect_class(
            LOCAL_BALKA_RUZHIMMASH_DETECT_MODEL_ADDRESS, yolov5_dir)

        print('reading and loading - BALKA_TIHVIN_DETECT_MODEL')
        self.balka_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_detect_class(
            LOCAL_BALKA_TIHVIN_DETECT_MODEL_ADDRESS,
            yolov5_dir)
        print('reading and loading - BALKA_ALTAI_DETECT_MODEL')
        if balka_detect_version == 2:
            self.balka_detect_objects[balka_prod.Balka_Prod.altai] = Balka_detect_class(
                LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS_2, yolov5_dir)
        elif balka_detect_version == 3:
            self.balka_detect_objects[balka_prod.Balka_Prod.altai] = Balka_detect_class(
                LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS_3, yolov5_dir)
        else:
            self.balka_detect_objects[balka_prod.Balka_Prod.altai] = Balka_detect_class(
                LOCAL_BALKA_ALTAI_DETECT_MODEL_ADDRESS, yolov5_dir)
        self.balka_text_detect_objects = {
            balka_prod.Balka_Prod.altai: None,
            balka_prod.Balka_Prod.begickaya: None,
            balka_prod.Balka_Prod.promlit: None,
            balka_prod.Balka_Prod.ruzhimmash: None,
            balka_prod.Balka_Prod.tihvin: None
        }
        print('reading and loading - BALKA_RUZHIMMASH_TEXT_DETECT_MODEL')
        self.balka_text_detect_objects[balka_prod.Balka_Prod.ruzhimmash] = Balka_text_detect_class(
            LOCAL_BALKA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.ruzhimmash)


        if balka_text_detect_version > 1:
            print('reading and loading - BALKA_ALTAI_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.altai] = Balka_text_detect_altai_class(
                LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_2, yolov5_dir, balka_prod.Balka_Prod.altai)
            if balka_text_detect_version > 2:
                print('reading and loading - BALKA_BEGICKAYA_TEXT_DETECT_MODEL')
                self.balka_text_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_text_detect_class(
                    LOCAL_BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS_2, yolov5_dir, balka_prod.Balka_Prod.begickaya)
                if balka_text_detect_version > 3:
                    print('reading and loading - BALKA_PROMLIT_TEXT_DETECT_MODEL')
                    self.balka_text_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_text_detect_class(
                        LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS_2, yolov5_dir, balka_prod.Balka_Prod.promlit)
                    if (balka_text_detect_version == 5) or (balka_text_detect_version == 7):
                        print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
                        self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                            LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS_2,
                            yolov5_dir, balka_prod.Balka_Prod.tihvin)
                    else:
                        print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
                        self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                            LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS,
                            yolov5_dir, balka_prod.Balka_Prod.tihvin)
                    if (balka_text_detect_version > 5):
                        print('reading and loading - BALKA_ALTAI_TEXT_DETECT_MODEL')
                        self.balka_text_detect_objects[balka_prod.Balka_Prod.altai] = Balka_text_detect_altai_class(
                            LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_3, yolov5_dir, balka_prod.Balka_Prod.altai)
                    else:
                        print('reading and loading - BALKA_ALTAI_TEXT_DETECT_MODEL')
                        self.balka_text_detect_objects[balka_prod.Balka_Prod.altai] = Balka_text_detect_altai_class(
                            LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_2, yolov5_dir, balka_prod.Balka_Prod.altai)

                else:
                    print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
                    self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                        LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS,
                        yolov5_dir, balka_prod.Balka_Prod.tihvin)
                    print('reading and loading - BALKA_PROMLIT_TEXT_DETECT_MODEL')
                    self.balka_text_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_text_detect_class(
                        LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.promlit)
            else:
                print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
                self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                    LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS,
                    yolov5_dir, balka_prod.Balka_Prod.tihvin)
                print('reading and loading - BALKA_BEGICKAYA_TEXT_DETECT_MODEL')
                self.balka_text_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_text_detect_class(
                    LOCAL_BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.begickaya)
                print('reading and loading - BALKA_PROMLIT_TEXT_DETECT_MODEL')
                self.balka_text_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_text_detect_class(
                    LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.promlit)
        else:
            print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                LOCAL_BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS,
                yolov5_dir, balka_prod.Balka_Prod.tihvin)
            print('reading and loading - BALKA_ALTAI_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.altai] = Balka_text_detect_class(
                LOCAL_BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.altai)
            print('reading and loading - BALKA_BEGICKAYA_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_text_detect_class(
                LOCAL_BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.begickaya)
            print('reading and loading - BALKA_PROMLIT_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_text_detect_class(
                LOCAL_BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.promlit)


    def balka_text_detect(self, img, balka_prod):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_balka' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rects = self.balka_detect_objects[balka_prod].balka_detect(initial_image_path)
        if rects == []:
            os.remove(initial_image_path)
            return img
        imgs = []
        for rect in rects:
            imgs.append(img[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        image_text_areas_half_balkas = self.balka_text_detect_objects[balka_prod].balka_text_detect(imgs)
        images_drawn_text_areas = self.balka_text_detect_objects[balka_prod].draw_text_areas_in_opencv(
            image_text_areas_half_balkas, imgs)
        if len(images_drawn_text_areas) == 1:
            new_img = images_drawn_text_areas[0]
        else:
            h0, w0 = images_drawn_text_areas[0].shape[:2]
            h1, w1 = images_drawn_text_areas[1].shape[:2]
            dsize = (w0 + w1, max(h0,h1))
            new_img = cv2.resize(images_drawn_text_areas[0],dsize)
            left_image = cv2.resize(images_drawn_text_areas[0], ( w0, max(h0,h1)))
            right_image = cv2.resize(images_drawn_text_areas[1], ( w1, max(h0,h1)))
            for i in range(max(h0,h1)):
                for j in range(w0):
                    new_img[i,j] = left_image[i,j]
                for k in range(w0,w0+w1):
                    new_img[i,k] = right_image[i,k - w0]
        os.remove(initial_image_path)
        return new_img

    def balka_text_detect_cuts(self, img, balka_prod):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_balka' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rects = self.balka_detect_objects[balka_prod].balka_detect(initial_image_path)
        if rects == []:
            os.remove(initial_image_path)
            return [], []
        imgs = []
        for rect in rects:
            imgs.append(img[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        image_text_areas_half_balkas = self.balka_text_detect_objects[balka_prod].balka_text_detect(imgs)
        os.remove(initial_image_path)
        return imgs, image_text_areas_half_balkas


class Text_recognize_result:
    def __init__(self, detail, number = None, prod = None, year = None):
        self.detail = detail
        self.number = number
        self.prod = prod
        self.year = year


from danila.danila_balka.danila_balka_classify.Danila_balka_classify import Danila_balka_classify
from danila.danila_balka.danila_balka_text_detect.danila_balka_text_detect import Danila_balka_text_detect
from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize import Danila_balka_text_recognize
from data.neuro.prods import BALKA_PRODS
from data.result.Text_cut_recognize_result import Text_cut_recognize_result
from data.result.Text_recognize_result import Text_recognize_result
from data.result.balka_prod import Balka_Prod


class Danila_balka_redone:
    def __init__(self, yolov5_dir, danila_balka_params):
        self.danila_balka_params = danila_balka_params
        self.danila_balka_classify = Danila_balka_classify(yolov5_dir, self.danila_balka_params.danila_balka_classify_params)
        self.danila_balka_text_detect = Danila_balka_text_detect(yolov5_dir,
                                                               self.danila_balka_params.danila_balka_text_detect_params)
        self.danila_balka_text_recognize = Danila_balka_text_recognize(yolov5_dir, self.danila_balka_params.danila_balka_text_recognize_params)


    def balka_classify(self,img, detail):
        balka_prod_conf = self.danila_balka_classify.balka_classify(img)
        if detail is None:
            detail_prod = Text_cut_recognize_result('balka', 1)
            det = Text_recognize_result(detail_prod)
        else:
            det = detail
        if balka_prod_conf.balka_prod != Balka_Prod.no_balka:
            text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
            det.prod = Text_cut_recognize_result(text_prod, balka_prod_conf.conf)
        return det

    def balka_text_detect(self, img):
        balka_prod_conf = self.danila_balka_classify.balka_classify(img)
        result_img = img
        if balka_prod_conf.balka_prod != Balka_Prod.no_balka:
            result_img = self.danila_balka_text_detect.balka_text_detect(img, balka_prod_conf.balka_prod)
        return result_img

    def balka_text_recognize(self, img, detail):
        balka_prod_conf = self.danila_balka_classify.balka_classify(img)
        if detail is None:
            detail_prod = Text_cut_recognize_result('balka', 1)
            det = Text_recognize_result(detail_prod)
        else:
            det = detail
        if balka_prod_conf.balka_prod != Balka_Prod.no_balka:
            text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
            det.prod = Text_cut_recognize_result(text_prod, balka_prod_conf.conf)
            img_cuts, img_text_areas_2_balkas = self.danila_balka_text_detect.balka_rext_detect_cuts(img, balka_prod_conf.balka_prod)

            res_labels = self.danila_balka_text_recognize.balka_text_recognize(
                balka_prod=balka_prod_conf.balka_prod,
                img_cuts=img_cuts,
                img_text_areas_2_balkas=img_text_areas_2_balkas)
            res_labels_prod_text, res_labels_prod_conf = res_labels['prod']
            if res_labels_prod_text != text_prod:
                if res_labels_prod_text == '1923':
                    text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
                    det.prod = Text_cut_recognize_result(res_labels_prod_text, balka_prod_conf.conf)
                    img_cut, img_text_areas = self.danila_balka_text_detect.balka_rext_detect_cuts(img,
                                                                                                 Balka_Prod.altai)
                    res_labels = self.danila_balka_text_recognize.balka_text_recognize(Balka_Prod.altai, img_cut,
                                                                                     img_text_areas)
                elif res_labels_prod_text == '12':
                    text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
                    det.prod = Text_cut_recognize_result(res_labels_prod_text, balka_prod_conf.conf)
                    img_cut, img_text_areas = self.danila_balka_text_detect.balka_rext_detect_cuts(img,
                                                                                                 Balka_Prod.begickaya)
                    res_labels = self.danila_balka_text_recognize.balka_text_recognize(Balka_Prod.begickaya, img_cut,
                                                                                     img_text_areas)
                elif res_labels_prod_text == '33':
                    text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
                    det.prod = Text_cut_recognize_result(res_labels_prod_text, balka_prod_conf.conf)
                    img_cut, img_text_areas = self.danila_balka_text_detect.balka_rext_detect_cuts(img,
                                                                                                   Balka_Prod.promlit)
                    res_labels = self.danila_balka_text_recognize.balka_text_recognize(Balka_Prod.promlit, img_cut,
                                                                                       img_text_areas)
                elif res_labels_prod_text == '1275':
                    text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
                    det.prod = Text_cut_recognize_result(res_labels_prod_text, balka_prod_conf.conf)
                    img_cut, img_text_areas = self.danila_balka_text_detect.balka_rext_detect_cuts(img,
                                                                                                   Balka_Prod.ruzhimmash)
                    res_labels = self.danila_balka_text_recognize.balka_text_recognize(Balka_Prod.ruzhimmash, img_cut,
                                                                                       img_text_areas)
                elif res_labels_prod_text == '1378':
                    text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
                    det.prod = Text_cut_recognize_result(res_labels_prod_text, balka_prod_conf.conf)
                    img_cut, img_text_areas = self.danila_balka_text_detect.balka_rext_detect_cuts(img,
                                                                                                   Balka_Prod.tihvin)
                    res_labels = self.danila_balka_text_recognize.balka_text_recognize(Balka_Prod.tihvin, img_cut,
                                                                                       img_text_areas)
            res_labels_number_text, res_labels_number_conf = res_labels['number']
            det.number = Text_cut_recognize_result(res_labels_number_text, res_labels_number_conf)
            res_labels_year_text, res_labels_year_conf = res_labels['year']
            det.year = Text_cut_recognize_result(res_labels_year_text, res_labels_year_conf)
        return det


from danila.danila_rama.danila_rama_text_recognize.danila_rama_text_recognize_init import \
    Danila_rama_text_recognize_init
from danila.danila_rama.danila_rama_text_recognize.danila_rama_text_recognize_number_shift import \
    Danila_rama_text_recognize_number_shift
from danila.danila_rama.danila_rama_text_recognize.danila_rama_text_recognize_prod_redone import \
    Danila_rama_text_recognize_prod_redone
from danila.danila_rama.danila_rama_text_recognize.danila_rama_text_recognize_year_post import \
    Danila_rama_text_recognize_year_post


class Danila_rama_text_recognize:
    def __init__(self, yolov5_dir, danila_rama_text_recognize_params):
        danila_rama_text_recognize = Danila_rama_text_recognize_init(
            yolov5_dir=yolov5_dir,
            rama_text_recognize_version=danila_rama_text_recognize_params.rama_text_recognize_model_version
        )
        if danila_rama_text_recognize_params.prod_redone:
            danila_rama_text_recognize = Danila_rama_text_recognize_prod_redone(danila_rama_text_recognize)
        if danila_rama_text_recognize_params.number_shift:
            danila_rama_text_recognize = Danila_rama_text_recognize_number_shift(danila_rama_text_recognize)
        if danila_rama_text_recognize_params.year_post:
            danila_rama_text_recognize = Danila_rama_text_recognize_year_post(danila_rama_text_recognize)
        self.danila_rama_text_recognize = danila_rama_text_recognize


    def rama_text_recognize(self, rama_prod, img_cut, img_text_areas):
        return self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, img_text_areas)


RAMA_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/4xtNt0zftHFFVg'

RAMA_BEGICKAYA_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/dld9TFyEQGWBcw'
RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/42sWB5wc7iCZgQ'
RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/7lAoRbs6HUkLaA'
RAMA_ALTAI_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/q3FDt5wzdjCg7Q'
RAMA_ALTAI_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/klpSwrGiS9T0OQ'
RAMA_BALAKOVO_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/HlcViPdjY2hS5g'
RAMA_PROMLIT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/wRdF0UOdGoXlVQ'
RAMA_PROMLIT_DETECT_MODEL_ADDRESS_2 ='https://disk.yandex.ru/d/r631dRw9gzKQAQ'
RAMA_TIHVIN_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/8xJz_9PsUDH41Q'
RAMA_URALVAGON_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/QK7sDnJDisNJiQ'

RAMA_ALTAI_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/Pb4cNl3JxcUw6Q'
RAMA_ALTAI_TEXT_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/izG9zUipd3Ws5Q'
RAMA_BALAKOVO_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/bitLjF_wboq26Q'
RAMA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/nQaiB6YheVSc7Q'
RAMA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/lu11fYVSW64k6A'
RAMA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/wISY6tAbHNpg4g'
RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/B5brahwfJhy9mg'
RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/UZdPspStE0SAHA'
RAMA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/fbQL7mq1wV6ueA'
RAMA_URALVAGON_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/HZH6SabldzFSkQ'

# RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS = 'https://disk.yandex.ru/d/C2hDMsbTZMnL8Q' -7500_b7500_r
# TEXT_RECOGNIZE_BEGICKAYA_MODEL_ADDRESS = 'https://disk.yandex.ru/d/C2hDMsbTZMnL8Q'
RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS = 'https://disk.yandex.ru/d/GcI-mAtwpb2I-g' # 11000_b_11000_r
RAMA_TEXT_RECOGNIZE_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/hhvxqKMaSgFJMA' # 5000_a_11000_b_11000_r


RAMA_PROD_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/8FI8TaTwaCfbSA'
RAMA_PROD_CLASSIFY_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/P7rKubXWHJjgCQ'
RAMA_PROD_CLASSIFY_MODEL_ADDRESS_3 = 'https://disk.yandex.ru/d/41FTqiZhIW_Ytg'
VAGON_NUMBER_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/ZGS0sdk1syJmcQ'
VAGON_NUMBER_DETECT_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/3artWauJULaWXw'

VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS = 'https://disk.yandex.ru/d/eh714_PrjFyLHg'
VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/iZxGW6ZaXZDoQQ'
VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_3 = 'https://disk.yandex.ru/d/wd697DtHumhsjA'
VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_4 = 'https://disk.yandex.ru/d/4UqcWigCF5hCqw'

BALKA_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/B0sdDuJQnmR9ZA'

BALKA_ALTAI_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/7JLCBgAC3hH4jw'
BALKA_BEGICKAYA_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/NBIl4fwZkrQaZA'
BALKA_PROMLIT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/g-aW6tMTsrI9Iw'
BALKA_RUZHIMMASH_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/WVEld5fLliy-SA'
BALKA_TIHVIN_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/ItUmCGqM5p5j9g'

BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/E2AMY5EW1W3AsQ'
BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/BUmrhYaD08oGGw'
BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/dgJW0CcYQe2q0A'
BALKA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/M8Wz2z420XPpBw'
BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS = 'https://disk.yandex.ru/d/X0_WRNJfOreOsQ'

DETAIL_CLASSIFY_MODEL_ADDRESS = 'https://disk.yandex.ru/d/Xamk_OYzYAbGhg'
DETAIL_CLASSIFY_MODEL_ADDRESS_2 = 'https://disk.yandex.ru/d/_t4r3N7a2Uy3cA'
DETAIL_CLASSIFY_YOLO_MODEL_ADDRESS = 'https://disk.yandex.ru/d/2VNw___O_k0J_A'
DETAIL_CLASSIFY_BALKA_MODEL_ADDRESS = BALKA_PROMLIT_DETECT_MODEL_ADDRESS
DETAIL_CLASSIFY_RAMA_MODEL_ADDRESS = RAMA_BALAKOVO_DETECT_MODEL_ADDRESS
DETAIL_CLASSIFY_VAGON_MODEL_ADDRESS = VAGON_NUMBER_DETECT_MODEL_ADDRESS




from danila.danila_vagon.danila_vagon_text_detect_base import Danila_vagon_text_detect_base


class Danila_vagon_text_detect:
    def __init__(self, local, yolov5_dir, vagon_text_detect_params):
        self.danila_vagon_text_detect = Danila_vagon_text_detect_base(local, yolov5_dir, vagon_text_detect_params)

    def vagon_number_detect(self, img):
        return self.danila_vagon_text_detect.vagon_number_detect(img)

    def vagon_number_detect_rects(self, img):
        return self.danila_vagon_text_detect.vagon_number_detect_rects(img)


import json
import os

import cv2
import torch
import zipfile
import requests
from urllib.parse import urlencode
from data.result import Rect, Yolo_label_rect

"""module for detecting rama"""
class Rama_detect_class:
    """module for detecting rama"""

    # reads yolov5 taught model from yandex-disk and includes it in class example
    def __init__(self, local, model_path, yolo_path, sort_by_square = False):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        self.sort_by_square = sort_by_square
        if local:
            self.model = torch.hub.load(yolo_path, 'custom', model_path, source='local')

            # self.model.max_det = 1
        else:
            base_url = 'https://cloud-api.yandex.net/v1/disk/public/resources/download?'
            public_key = model_path  # Сюда вписываете вашу ссылку
            # Получаем загрузочную ссылку
            final_url = base_url + urlencode(dict(public_key=public_key))
            response = requests.get(final_url)
            download_url = response.json()['href']
            # Загружаем файл и сохраняем его
            download_response = requests.get(download_url)
            zip_path = 'rama_detect.zip'
            # print(download_response.content)
            with open(zip_path, 'wb') as f:
                f.write(download_response.content)

            with zipfile.ZipFile(zip_path, 'r') as zip_ref:
                zip_ref.extractall()
            weights_file_path = 'rama_detect.pt'
            self.model = torch.hub.load(yolo_path, 'custom', weights_file_path, source='local')
            # self.model.max_det = 1



    #получить JSON с результатами yolo
    def work_img(self, img_path):
        """get JSON with yolo_results from img from img_path"""
        results = self.model([img_path])
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        return res2

    # получить координаты прямоугольника с рамой
    def rama_detect(self, img_path):
        """get Rect object with rama coordinates from img from img_path"""
        json_res_pred = self.work_img(img_path)
        if self.sort_by_square:
            json_res = sorted(json_res_pred, key = lambda rama: (float(rama['xmax']) - float(rama['xmin']))*(float(rama['ymax']) - float(rama['ymin'])), reverse=True)
        else:
            json_res = json_res_pred
        rect = Rect.Rect.get_rect_from_yolo_json(json_res)
        return rect





from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize_altai_work import \
    Danila_balka_text_recognize_altai_work
from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize_model_3 import \
    Danila_balka_text_recognize_model_3
from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize_model_4 import \
    Danila_balka_text_recognize_model_4
from danila.danila_balka.danila_balka_text_recognize.danila_balka_text_recognize_work import \
    Danila_balka_text_recognize_work
from data.result.balka_prod import Balka_Prod


class Danila_balka_text_recognize_init_5:
    def __init__(self, yolov5_dir):
        danila_balka_text_recognize_model_4 = Danila_balka_text_recognize_model_4()
        model_4 = danila_balka_text_recognize_model_4.get_model(yolov5_dir)
        model_altai_4 = danila_balka_text_recognize_model_4.get_model_altai(yolov5_dir)
        danila_balka_text_recognize_model_3 = Danila_balka_text_recognize_model_3()
        model_3 = danila_balka_text_recognize_model_3.get_model(yolov5_dir)
        model_altai_3 = danila_balka_text_recognize_model_3.get_model_altai(yolov5_dir)
        self.prod_models = {}
        for balka_prod in Balka_Prod:
            if balka_prod == Balka_Prod.altai:
                self.prod_models[balka_prod] = Danila_balka_text_recognize_altai_work(
                    text_recognize_model=model_altai_3
                )
            else:
                self.prod_models[balka_prod] = Danila_balka_text_recognize_work(
                    text_recognize_model=model_4,
                    prod_coefficients=danila_balka_text_recognize_model_4.get_prod_coefficients(balka_prod)
                )

    def balka_text_recognize(self, balka_prod, img_cut, image_text_areas):
        return self.prod_models[balka_prod].balka_text_recognize(balka_prod, img_cut, image_text_areas)

